@@ -6,6 +6,7 @@ from . import (
     lazy,
     likelihoods,
     means,
+    metrics,
     mlls,
     models,
     optim,
@@ -24,6 +25,7 @@ from .functions import (  # Deprecated
     log_normal_cdf,
     logdet,
     matmul,
+    pivoted_cholesky,
     root_decomposition,
     root_inv_decomposition,
 )
@@ -31,7 +33,7 @@ from .lazy import cat, delazify, lazify
 from .mlls import ExactMarginalLogLikelihood
 from .module import Module
 
-__version__ = "1.5.0"
+__version__ = "1.7.0"
 
 __all__ = [
     # Submodules
@@ -40,6 +42,7 @@ __all__ = [
     "lazy",
     "likelihoods",
     "means",
+    "metrics",
     "mlls",
     "models",
     "optim",
@@ -62,6 +65,7 @@ __all__ = [
     "logdet",
     "log_normal_cdf",
     "matmul",
+    "pivoted_cholesky",
     "root_decomposition",
     "root_inv_decomposition",
     # Context managers
