@@ -62,6 +62,15 @@ class MultitaskMultivariateNormal(MultivariateNormal):
             mean_mvn = mean.transpose(-1, -2).reshape(*mean.shape[:-2], -1)
         super().__init__(mean=mean_mvn, covariance_matrix=covariance_matrix, validate_args=validate_args)
 
+    @property
+    def base_sample_shape(self):
+        """
+        Returns the shape of a base sample (without batching) that is used to
+        generate a single sample.
+        """
+        base_sample_shape = self.event_shape
+        return base_sample_shape
+
     @property
     def event_shape(self):
         return self._output_shape[-2:]
