@@ -1,10 +1,12 @@
 #!/usr/bin/env python3
 
 import math
+from typing import Optional
 
 import torch
 
-from ..constraints import Positive
+from ..constraints import Interval, Positive
+from ..priors import Prior
 from .kernel import Kernel
 
 
@@ -56,7 +58,12 @@ class CosineKernel(Kernel):
 
     is_stationary = True
 
-    def __init__(self, period_length_prior=None, period_length_constraint=None, **kwargs):
+    def __init__(
+        self,
+        period_length_prior: Optional[Prior] = None,
+        period_length_constraint: Optional[Interval] = None,
+        **kwargs,
+    ):
         super(CosineKernel, self).__init__(**kwargs)
 
         self.register_parameter(
@@ -67,6 +74,8 @@ class CosineKernel(Kernel):
             period_length_constraint = Positive()
 
         if period_length_prior is not None:
+            if not isinstance(period_length_prior, Prior):
+                raise TypeError("Expected gpytorch.priors.Prior but got " + type(period_length_prior).__name__)
             self.register_prior(
                 "period_length_prior",
                 period_length_prior,
