@@ -1,5 +1,9 @@
 #!/usr/bin/env python3
 
+from typing import Callable
+
+import torch
+
 from .kernel import Kernel
 
 
@@ -20,7 +24,11 @@ class DistributionalInputKernel(Kernel):
     """
     has_lengthscale = True
 
-    def __init__(self, distance_function, **kwargs):
+    def __init__(
+        self,
+        distance_function: Callable,
+        **kwargs,
+    ):
         super(DistributionalInputKernel, self).__init__(**kwargs)
         if distance_function is None:
             raise NotImplementedError("DistributionalInputKernel requires a distance function.")
@@ -34,4 +42,7 @@ class DistributionalInputKernel(Kernel):
         if not diag:
             return res
         else:
-            return res.diag()
+            if torch.is_tensor(res):
+                return res.diagonal(dim1=-1, dim2=-2)
+            else:
+                return res.diag()  # For LazyTensor
