@@ -2,11 +2,15 @@
 
 import copy
 import math
+from typing import Optional, Tuple
 
 import torch
+from torch import Tensor
 
+from .. import settings
 from ..distributions import MultivariateNormal
 from ..lazy import DiagLazyTensor, LowRankRootAddedDiagLazyTensor, LowRankRootLazyTensor, MatmulLazyTensor, delazify
+from ..likelihoods import Likelihood
 from ..mlls import InducingPointKernelAddedLossTerm
 from ..models import exact_prediction_strategies
 from ..utils.cholesky import psd_safe_cholesky
@@ -14,7 +18,13 @@ from .kernel import Kernel
 
 
 class InducingPointKernel(Kernel):
-    def __init__(self, base_kernel, inducing_points, likelihood, active_dims=None):
+    def __init__(
+        self,
+        base_kernel: Kernel,
+        inducing_points: Tensor,
+        likelihood: Likelihood,
+        active_dims: Optional[Tuple[int, ...]] = None,
+    ):
         super(InducingPointKernel, self).__init__(active_dims=active_dims)
         self.base_kernel = base_kernel
         self.likelihood = likelihood
@@ -61,7 +71,7 @@ class InducingPointKernel(Kernel):
             covar = LowRankRootLazyTensor(k_ux1.matmul(self._inducing_inv_root))
 
             # Diagonal correction for predictive posterior
-            if not self.training:
+            if not self.training and settings.sgpr_diagonal_correction.on():
                 correction = (self.base_kernel(x1, x2, diag=True) - covar.diag()).clamp(0, math.inf)
                 covar = LowRankRootAddedDiagLazyTensor(covar, DiagLazyTensor(correction))
         else:
