@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import math
+from typing import Optional
 
 import torch
 
@@ -79,7 +80,7 @@ class MaternKernel(Kernel):
 
     has_lengthscale = True
 
-    def __init__(self, nu=2.5, **kwargs):
+    def __init__(self, nu: Optional[float] = 2.5, **kwargs):
         if nu not in {0.5, 1.5, 2.5}:
             raise RuntimeError("nu expected to be 0.5, 1.5, or 2.5")
         super(MaternKernel, self).__init__(**kwargs)
@@ -106,8 +107,8 @@ class MaternKernel(Kernel):
             elif self.nu == 1.5:
                 constant_component = (math.sqrt(3) * distance).add(1)
             elif self.nu == 2.5:
-                constant_component = (math.sqrt(5) * distance).add(1).add(5.0 / 3.0 * distance ** 2)
+                constant_component = (math.sqrt(5) * distance).add(1).add(5.0 / 3.0 * distance**2)
             return constant_component * exp_component
-        return MaternCovariance().apply(
+        return MaternCovariance.apply(
             x1, x2, self.lengthscale, self.nu, lambda x1, x2: self.covar_dist(x1, x2, **params)
         )
