@@ -7,7 +7,7 @@ import torch
 from torch import Tensor
 
 from .. import settings
-from ..utils import broadcasting, pivoted_cholesky
+from ..utils import broadcasting
 from ..utils.memoize import cached
 from ..utils.warnings import NumericalWarning
 from .diag_lazy_tensor import ConstantDiagLazyTensor, DiagLazyTensor
@@ -57,15 +57,15 @@ class AddedDiagLazyTensor(SumLazyTensor):
         return torch.addcmul(self._lazy_tensor._matmul(rhs), self._diag_tensor._diag.unsqueeze(-1), rhs)
 
     def add_diag(self, added_diag):
-        return AddedDiagLazyTensor(self._lazy_tensor, self._diag_tensor.add_diag(added_diag))
+        return self.__class__(self._lazy_tensor, self._diag_tensor.add_diag(added_diag))
 
     def __add__(self, other):
         from .diag_lazy_tensor import DiagLazyTensor
 
         if isinstance(other, DiagLazyTensor):
-            return AddedDiagLazyTensor(self._lazy_tensor, self._diag_tensor + other)
+            return self.__class__(self._lazy_tensor, self._diag_tensor + other)
         else:
-            return AddedDiagLazyTensor(self._lazy_tensor + other, self._diag_tensor)
+            return self.__class__(self._lazy_tensor + other, self._diag_tensor)
 
     def _preconditioner(self):
         r"""
@@ -97,7 +97,7 @@ class AddedDiagLazyTensor(SumLazyTensor):
         # Through matrix determinant lemma, log |L L^T + D| reduces down to 2 log |R|
         if self._q_cache is None:
             max_iter = settings.max_preconditioner_size.value()
-            self._piv_chol_self = pivoted_cholesky.pivoted_cholesky(self._lazy_tensor, max_iter)
+            self._piv_chol_self = self._lazy_tensor.pivoted_cholesky(rank=max_iter)
             if torch.any(torch.isnan(self._piv_chol_self)).item():
                 warnings.warn(
                     "NaNs encountered in preconditioner computation. Attempting to continue without preconditioning.",
@@ -136,7 +136,9 @@ class AddedDiagLazyTensor(SumLazyTensor):
     def _init_cache_for_constant_diag(self, eye, batch_shape, n, k):
         # We can factor out the noise for for both QR and solves.
         self._noise = self._noise.narrow(-2, 0, 1)
-        self._q_cache, self._r_cache = torch.qr(torch.cat((self._piv_chol_self, self._noise.sqrt() * eye), dim=-2))
+        self._q_cache, self._r_cache = torch.linalg.qr(
+            torch.cat((self._piv_chol_self, self._noise.sqrt() * eye), dim=-2)
+        )
         self._q_cache = self._q_cache[..., :n, :]
 
         # Use the matrix determinant lemma for the logdet, using the fact that R'R = L_k'L_k + s*I
@@ -146,7 +148,9 @@ class AddedDiagLazyTensor(SumLazyTensor):
 
     def _init_cache_for_non_constant_diag(self, eye, batch_shape, n):
         # With non-constant diagonals, we cant factor out the noise as easily
-        self._q_cache, self._r_cache = torch.qr(torch.cat((self._piv_chol_self / self._noise.sqrt(), eye), dim=-2))
+        self._q_cache, self._r_cache = torch.linalg.qr(
+            torch.cat((self._piv_chol_self / self._noise.sqrt(), eye), dim=-2)
+        )
         self._q_cache = self._q_cache[..., :n, :] / self._noise.sqrt()
 
         # Use the matrix determinant lemma for the logdet, using the fact that R'R = L_k'L_k + s*I
@@ -168,3 +172,17 @@ class AddedDiagLazyTensor(SumLazyTensor):
             evals = evals_ + self._diag_tensor.diag()
             return evals, evecs
         return super()._symeig(eigenvectors=eigenvectors)
+
+    def evaluate_kernel(self):
+        """
+        Overriding this is currently necessary to allow for subclasses of AddedDiagLT to be created. For example,
+        consider the following:
+
+            >>> covar1 = covar_module(x).add_diag(torch.tensor(1.)).evaluate_kernel()
+            >>> covar2 = covar_module(x).evaluate_kernel().add_diag(torch.tensor(1.))
+
+        Unless we override this method (or find a better solution), covar1 and covar2 might not be the same type.
+        In particular, covar1 would *always* be a standard AddedDiagLazyTensor, but covar2 might be a subtype.
+        """
+        added_diag_lazy_tsr = self.representation_tree()(*self.representation())
+        return added_diag_lazy_tsr._lazy_tensor + added_diag_lazy_tsr._diag_tensor
