@@ -3,6 +3,7 @@
 import math
 import warnings
 from abc import ABC, abstractmethod
+from copy import deepcopy
 from typing import Optional, Tuple
 
 import torch
@@ -14,11 +15,12 @@ from .. import settings, utils
 from ..functions._diagonalization import Diagonalization
 from ..functions._inv_matmul import InvMatmul
 from ..functions._inv_quad import InvQuad
-from ..functions._inv_quad_log_det import InvQuadLogDet
+from ..functions._inv_quad_logdet import InvQuadLogdet
 from ..functions._matmul import Matmul
+from ..functions._pivoted_cholesky import PivotedCholesky
 from ..functions._root_decomposition import RootDecomposition
 from ..functions._sqrt_inv_matmul import SqrtInvMatmul
-from ..utils.broadcasting import _matmul_broadcast_shape, _mul_broadcast_shape
+from ..utils.broadcasting import _matmul_broadcast_shape, _mul_broadcast_shape, _to_helper
 from ..utils.cholesky import psd_safe_cholesky
 from ..utils.deprecation import _deprecate_renamed_methods
 from ..utils.errors import CachingError
@@ -26,10 +28,11 @@ from ..utils.getitem import _compute_getitem_size, _convert_indices_to_tensors,
 from ..utils.lanczos import _postprocess_lanczos_root_inv_decomp
 from ..utils.memoize import _is_in_cache_ignore_all_args, _is_in_cache_ignore_args, add_to_cache, cached, pop_from_cache
 from ..utils.pinverse import stable_pinverse
-from ..utils.pivoted_cholesky import pivoted_cholesky
 from ..utils.warnings import NumericalWarning
 from .lazy_tensor_representation_tree import LazyTensorRepresentationTree
 
+_TYPES_DICT = {torch.float: "float", torch.half: "half", torch.double: "double"}
+
 
 class LazyTensor(ABC):
     r"""
@@ -242,7 +245,13 @@ class LazyTensor(ABC):
         # Construct interpolated LazyTensor
         from . import InterpolatedLazyTensor
 
-        res = InterpolatedLazyTensor(self, row_interp_indices, row_interp_values, col_interp_indices, col_interp_values)
+        res = InterpolatedLazyTensor(
+            self,
+            row_interp_indices,
+            row_interp_values,
+            col_interp_indices,
+            col_interp_values,
+        )
         return res._getitem(row_index, col_index, *batch_indices)
 
     def _unsqueeze_batch(self, dim):
@@ -315,7 +324,11 @@ class LazyTensor(ABC):
 
         res = (
             InterpolatedLazyTensor(
-                base_lazy_tensor, row_interp_indices, row_interp_values, col_interp_indices, col_interp_values
+                base_lazy_tensor,
+                row_interp_indices,
+                row_interp_values,
+                col_interp_indices,
+                col_interp_values,
             )
             .evaluate()
             .squeeze(-2)
@@ -404,8 +417,8 @@ class LazyTensor(ABC):
         Returns:
             (TriangularLazyTensor) Cholesky factor
         """
-        from .triangular_lazy_tensor import TriangularLazyTensor
         from .keops_lazy_tensor import KeOpsLazyTensor
+        from .triangular_lazy_tensor import TriangularLazyTensor
 
         evaluated_kern_mat = self.evaluate_kernel()
 
@@ -505,8 +518,8 @@ class LazyTensor(ABC):
         Returns:
             :obj:`gpytorch.lazy.LazyTensor`
         """
-        from .non_lazy_tensor import NonLazyTensor
         from .mul_lazy_tensor import MulLazyTensor
+        from .non_lazy_tensor import NonLazyTensor
 
         self = self.evaluate_kernel()
         other = other.evaluate_kernel()
@@ -515,7 +528,10 @@ class LazyTensor(ABC):
         else:
             left_lazy_tensor = self if self._root_decomposition_size() < other._root_decomposition_size() else other
             right_lazy_tensor = other if left_lazy_tensor is self else self
-            return MulLazyTensor(left_lazy_tensor.root_decomposition(), right_lazy_tensor.root_decomposition())
+            return MulLazyTensor(
+                left_lazy_tensor.root_decomposition(),
+                right_lazy_tensor.root_decomposition(),
+            )
 
     def _preconditioner(self):
         """
@@ -556,7 +572,10 @@ class LazyTensor(ABC):
                 shape = list(roots.shape)
                 shape[dim] = 1
                 extra_root = torch.full(
-                    shape, dtype=self.dtype, device=self.device, fill_value=(1.0 / math.sqrt(self.size(-2)))
+                    shape,
+                    dtype=self.dtype,
+                    device=self.device,
+                    fill_value=(1.0 / math.sqrt(self.size(-2))),
                 )
                 roots = torch.cat([roots, extra_root], dim)
                 num_batch += 1
@@ -696,8 +715,8 @@ class LazyTensor(ABC):
         Args:
             - diag (Scalar Tensor)
         """
-        from .diag_lazy_tensor import ConstantDiagLazyTensor, DiagLazyTensor
         from .added_diag_lazy_tensor import AddedDiagLazyTensor
+        from .diag_lazy_tensor import ConstantDiagLazyTensor, DiagLazyTensor
 
         if not self.is_square:
             raise RuntimeError("add_diag only defined for square matrices")
@@ -732,7 +751,14 @@ class LazyTensor(ABC):
         diag = torch.tensor(jitter_val, dtype=self.dtype, device=self.device)
         return self.add_diag(diag)
 
-    def cat_rows(self, cross_mat, new_mat, generate_roots=True, **root_decomp_kwargs):
+    def cat_rows(
+        self,
+        cross_mat,
+        new_mat,
+        generate_roots=True,
+        generate_inv_roots=True,
+        **root_decomp_kwargs,
+    ):
         """
         Concatenates new rows and columns to the matrix that this LazyTensor represents, e.g.
         C = [A B^T; B D]. where A is the existing lazy tensor, and B (cross_mat) and D (new_mat)
@@ -759,8 +785,10 @@ class LazyTensor(ABC):
                 If :math:`A` is n x n, then this matrix should be n x k.
             new_mat (:obj:`torch.tensor`): the matrix :math:`D` we are appending to the matrix :math:`A`.
                 If :math:`B` is n x k, then this matrix should be k x k.
-            generate_roots (:obj:`bool`): whether to generate the root decomposition of :math:`A` even if it
-                has not been created yet.
+            generate_roots (:obj:`bool`): whether to generate the root
+                decomposition of :math:`A` even if it has not been created yet.
+            generate_inv_roots (:obj:`bool`): whether to generate the root inv
+                decomposition of :math:`A` even if it has not been created yet.
 
         Returns:
             :obj:`LazyTensor`: concatenated lazy tensor with the new rows and columns.
@@ -770,6 +798,11 @@ class LazyTensor(ABC):
         from .root_lazy_tensor import RootLazyTensor
         from .triangular_lazy_tensor import TriangularLazyTensor
 
+        if not generate_roots and generate_inv_roots:
+            warnings.warn(
+                "root_inv_decomposition is only generated when " "root_decomposition is generated.",
+                UserWarning,
+            )
         B_, B = cross_mat, lazify(cross_mat)
         D = lazify(new_mat)
         batch_shape = B.shape[:-2]
@@ -780,19 +813,23 @@ class LazyTensor(ABC):
             A = self
 
         # form matrix C = [A B; B^T D], where A = self, B = cross_mat, D = new_mat
-        upper_row = CatLazyTensor(A, B, dim=-2)
-        lower_row = CatLazyTensor(B.transpose(-1, -2), D, dim=-2)
-        new_lazy_tensor = CatLazyTensor(upper_row, lower_row, dim=-1)
+        upper_row = CatLazyTensor(A, B, dim=-2, output_device=A.device)
+        lower_row = CatLazyTensor(B.transpose(-1, -2), D, dim=-2, output_device=A.device)
+        new_lazy_tensor = CatLazyTensor(upper_row, lower_row, dim=-1, output_device=A.device)
 
         # if the old lazy tensor does not have either a root decomposition or a root inverse decomposition
         # don't create one
-        does_not_have_roots = any(
-            _is_in_cache_ignore_args(self, key) for key in ("root_inv_decomposition", "root_inv_decomposition")
+        has_roots = any(
+            _is_in_cache_ignore_args(self, key)
+            for key in (
+                "root_decomposition",
+                "root_inv_decomposition",
+            )
         )
-        if not generate_roots and not does_not_have_roots:
+        if not generate_roots and not has_roots:
             return new_lazy_tensor
 
-        # Get compomnents for new root Z = [E 0; F G]
+        # Get components for new root Z = [E 0; F G]
         E = self.root_decomposition(**root_decomp_kwargs).root  # E = L, LL^T = A
         m, n = E.shape[-2:]
         R = self.root_inv_decomposition().root.evaluate()  # RR^T = A^{-1} (this is fast if L is triangular)
@@ -806,20 +843,24 @@ class LazyTensor(ABC):
         new_root[..., :m, :n] = E.evaluate()
         new_root[..., m:, : lower_left.shape[-1]] = lower_left
         new_root[..., m:, n : (n + schur_root.shape[-1])] = schur_root
-
-        if isinstance(E, TriangularLazyTensor) and isinstance(schur_root, TriangularLazyTensor):
-            # make sure these are actually upper triangular
-            if getattr(E, "upper", False) or getattr(schur_root, "upper", False):
-                raise NotImplementedError
-            # in this case we know new_root is triangular as well
-            new_root = TriangularLazyTensor(new_root)
-            new_inv_root = new_root.inverse().transpose(-1, -2)
-        else:
-            # otherwise we use the pseudo-inverse of Z as new inv root
-            new_inv_root = stable_pinverse(new_root).transpose(-2, -1)
+        if generate_inv_roots:
+            if isinstance(E, TriangularLazyTensor) and isinstance(schur_root, TriangularLazyTensor):
+                # make sure these are actually upper triangular
+                if getattr(E, "upper", False) or getattr(schur_root, "upper", False):
+                    raise NotImplementedError
+                # in this case we know new_root is triangular as well
+                new_root = TriangularLazyTensor(new_root)
+                new_inv_root = new_root.inverse().transpose(-1, -2)
+            else:
+                # otherwise we use the pseudo-inverse of Z as new inv root
+                new_inv_root = stable_pinverse(new_root).transpose(-2, -1)
+            add_to_cache(
+                new_lazy_tensor,
+                "root_inv_decomposition",
+                RootLazyTensor(lazify(new_inv_root)),
+            )
 
         add_to_cache(new_lazy_tensor, "root_decomposition", RootLazyTensor(lazify(new_root)))
-        add_to_cache(new_lazy_tensor, "root_inv_decomposition", RootLazyTensor(lazify(new_inv_root)))
 
         return new_lazy_tensor
 
@@ -861,7 +902,8 @@ class LazyTensor(ABC):
             new_lazy_tensor = self + lazify(low_rank_mat.matmul(low_rank_mat.transpose(-1, -2)))
         else:
             new_lazy_tensor = SumLazyTensor(
-                *self.lazy_tensors, lazify(low_rank_mat.matmul(low_rank_mat.transpose(-1, -2)))
+                *self.lazy_tensors,
+                lazify(low_rank_mat.matmul(low_rank_mat.transpose(-1, -2))),
             )
 
             # return as a nonlazy tensor if small enough to reduce memory overhead
@@ -870,10 +912,8 @@ class LazyTensor(ABC):
 
         # if the old lazy tensor does not have either a root decomposition or a root inverse decomposition
         # don't create one
-        does_not_have_roots = any(
-            _is_in_cache_ignore_args(self, key) for key in ("root_decomposition", "root_inv_decomposition")
-        )
-        if not generate_roots and not does_not_have_roots:
+        has_roots = any(_is_in_cache_ignore_args(self, key) for key in ("root_decomposition", "root_inv_decomposition"))
+        if not generate_roots and not has_roots:
             return new_lazy_tensor
 
         # we are going to compute the following
@@ -895,7 +935,7 @@ class LazyTensor(ABC):
         # we want the root decomposition of I_r + U S^2 U^T but S is q so we need to pad.
         one_padding = torch.ones(*S.shape[:-1], U.shape[-2] - S.shape[-1], device=S.device, dtype=S.dtype)
         # the non zero eigenvalues get updated by S^2 + 1, so we take the square root.
-        root_S_plus_identity = (S ** 2 + 1.0) ** 0.5
+        root_S_plus_identity = (S**2 + 1.0) ** 0.5
         # pad the nonzero eigenvalues with the ones
         #######
         # \tilde{S} = \left(((S^2 + 1)^{0.5}; 0
@@ -911,7 +951,12 @@ class LazyTensor(ABC):
             updated_root = torch.cat(
                 (
                     current_root.evaluate(),
-                    torch.zeros(*current_root.shape[:-1], 1, device=current_root.device, dtype=current_root.dtype),
+                    torch.zeros(
+                        *current_root.shape[:-1],
+                        1,
+                        device=current_root.device,
+                        dtype=current_root.dtype,
+                    ),
                 ),
                 dim=-1,
             )
@@ -1063,19 +1108,7 @@ class LazyTensor(ABC):
         """
         This method operates identically to :func:`torch.Tensor.double`.
         """
-        new_args = []
-        new_kwargs = {}
-        for arg in self._args:
-            if hasattr(arg, "double"):
-                new_args.append(arg.double())
-            else:
-                new_args.append(arg)
-        for name, val in self._kwargs.items():
-            if hasattr(val, "double"):
-                new_kwargs[name] = val.double()
-            else:
-                new_kwargs[name] = val
-        return self.__class__(*new_args, **new_kwargs)
+        return self.type(torch.double)
 
     @property
     def dtype(self):
@@ -1122,6 +1155,18 @@ class LazyTensor(ABC):
         """
         return self.representation_tree()(*self.representation())
 
+    def float(self, device_id=None):
+        """
+        This method operates identically to :func:`torch.Tensor.float`.
+        """
+        return self.type(torch.float)
+
+    def half(self, device_id=None):
+        """
+        This method operates identically to :func:`torch.Tensor.half`.
+        """
+        return self.type(torch.half)
+
     def inv_matmul(self, right_tensor, left_tensor=None):
         r"""
         Computes a linear solve (w.r.t self = :math:`A`) with several right hand sides :math:`R`.
@@ -1171,7 +1216,13 @@ class LazyTensor(ABC):
         if left_tensor is None:
             return func.apply(self.representation_tree(), False, right_tensor, *self.representation())
         else:
-            return func.apply(self.representation_tree(), True, left_tensor, right_tensor, *self.representation())
+            return func.apply(
+                self.representation_tree(),
+                True,
+                left_tensor,
+                right_tensor,
+                *self.representation(),
+            )
 
     def inv_quad(self, tensor, reduce_inv_quad=True):
         """
@@ -1238,7 +1289,19 @@ class LazyTensor(ABC):
                     will_need_cholesky = False
             if will_need_cholesky:
                 cholesky = CholLazyTensor(TriangularLazyTensor(self.cholesky()))
-            return cholesky.inv_quad_logdet(inv_quad_rhs=inv_quad_rhs, logdet=logdet, reduce_inv_quad=reduce_inv_quad)
+            return cholesky.inv_quad_logdet(
+                inv_quad_rhs=inv_quad_rhs,
+                logdet=logdet,
+                reduce_inv_quad=reduce_inv_quad,
+            )
+
+        # Short circuit to inv_quad function if we're not computing logdet
+        if not logdet:
+            if inv_quad_rhs is None:
+                raise RuntimeError("Either `inv_quad_rhs` or `logdet` must be specifed.")
+            return self.inv_quad(inv_quad_rhs, reduce_inv_quad=reduce_inv_quad), torch.zeros(
+                [], dtype=self.dtype, device=self.device
+            )
 
         # Default: use modified batch conjugate gradients to compute these terms
         # See NeurIPS 2018 paper: https://arxiv.org/abs/1809.11165
@@ -1272,22 +1335,34 @@ class LazyTensor(ABC):
         if inv_quad_rhs is not None:
             args = [inv_quad_rhs] + list(args)
 
+        preconditioner, precond_lt, logdet_p = self._preconditioner()
+        if precond_lt is None:
+            from ..lazy.identity_lazy_tensor import IdentityLazyTensor
+
+            precond_lt = IdentityLazyTensor(
+                diag_shape=self.size(-1),
+                batch_shape=self.batch_shape,
+                dtype=self.dtype,
+                device=self.device,
+            )
+            logdet_p = 0.0
+
+        precond_args = precond_lt.representation()
         probe_vectors, probe_vector_norms = self._probe_vectors_and_norms()
 
-        func = InvQuadLogDet.apply
-
-        inv_quad_term, logdet_term = func(
+        func = InvQuadLogdet.apply
+        inv_quad_term, pinvk_logdet = func(
             self.representation_tree(),
-            self.dtype,
-            self.device,
-            self.matrix_shape,
-            self.batch_shape,
+            precond_lt.representation_tree(),
+            preconditioner,
+            len(precond_args),
             (inv_quad_rhs is not None),
-            logdet,
             probe_vectors,
             probe_vector_norms,
-            *args,
+            *(list(args) + list(precond_args)),
         )
+        logdet_term = pinvk_logdet
+        logdet_term = logdet_term + logdet_p
 
         if inv_quad_term.numel() and reduce_inv_quad:
             inv_quad_term = inv_quad_term.sum(-1)
@@ -1333,6 +1408,23 @@ class LazyTensor(ABC):
 
         return Matmul.apply(self.representation_tree(), other, *self.representation())
 
+    def rmatmul(self, other):
+        """
+        Multiplies a matrix by self.
+
+        Args:
+            other (:obj:`torch.tensor`): Matrix or vector to multiply with. Can be either a :obj:`torch.tensor`
+                or a :obj:`gpytorch.lazy.LazyTensor`.
+
+        Returns:
+            :obj:`torch.tensor`: Tensor or LazyTensor containing the result of the matrix multiplication :math:`MK`,
+            where :math:`M` is the (batched) matrix input to this method, and :math:`K` is the (batched) matrix that
+            this :obj:`gpytorch.lazy.LazyTensor` represents.
+        """
+        if other.ndim == 1:
+            return self.transpose(-1, -2).matmul(other)
+        return self.transpose(-1, -2).matmul(other.transpose(-1, -2)).transpose(-1, -2)
+
     @property
     def matrix_shape(self):
         """
@@ -1354,8 +1446,8 @@ class LazyTensor(ABC):
             :obj:`gpytorch.lazy.ConstantMulLazyTensor`. If other was
             another matrix, this will likely be a :obj:`gpytorch.lazy.MulLazyTensor`.
         """
-        from .zero_lazy_tensor import ZeroLazyTensor
         from .non_lazy_tensor import lazify
+        from .zero_lazy_tensor import ZeroLazyTensor
 
         if isinstance(other, ZeroLazyTensor):
             return other
@@ -1419,6 +1511,37 @@ class LazyTensor(ABC):
 
         return self._permute_batch(*dims[:-2])
 
+    def pivoted_cholesky(self, rank, error_tol=None, return_pivots=False):
+        r"""
+        Performs a partial pivoted Cholesky factorization of the (positive definite) LazyTensor.
+        :math:`\mathbf L \mathbf L^\top = \mathbf K`.
+        The partial pivoted Cholesky factor :math:`\mathbf L \in \mathbb R^{N \times \text{rank}}`
+        forms a low rank approximation to the LazyTensor.
+
+        The pivots are selected greedily, corresponding to the maximum diagonal element in the
+        residual after each Cholesky iteration. See `Harbrecht et al., 2012`_.
+
+        :param int rank: The size of the partial pivoted Cholesky factor.
+        :param error_tol: Defines an optional stopping criterion.
+            If the residual of the factorization is less than :attr:`error_tol`, then the
+            factorization will exit early. This will result in a :math:`\leq \text{ rank}` factor.
+        :type error_tol: float, optional
+        :param bool return_pivots: (default: False) Whether or not to return the pivots alongside
+            the partial pivoted Cholesky factor.
+        :return: the `... x N x rank` factor (and optionally the `... x N` pivots)
+        :rtype: torch.Tensor or tuple(torch.Tensor, torch.Tensor)
+
+        .. _Harbrecht et al., 2012:
+            https://www.sciencedirect.com/science/article/pii/S0168927411001814
+        """
+        func = PivotedCholesky.apply
+        res, pivots = func(self.representation_tree(), rank, error_tol, *self.representation())
+
+        if return_pivots:
+            return res, pivots
+        else:
+            return res
+
     def prod(self, dim=None):
         """
         For a `b x n x m` LazyTensor, compute the product over the batch dimension.
@@ -1608,6 +1731,7 @@ class LazyTensor(ABC):
         This can be used for sampling from a Gaussian distribution, or for obtaining a
         low-rank version of a matrix
         """
+        from . import lazify
         from .chol_lazy_tensor import CholLazyTensor
         from .root_lazy_tensor import RootLazyTensor
 
@@ -1630,13 +1754,14 @@ class LazyTensor(ABC):
                 return CholLazyTensor(res)
             except RuntimeError as e:
                 warnings.warn(
-                    f"Runtime Error when computing Cholesky decomposition: {e}. Using symeig method.", NumericalWarning,
+                    f"Runtime Error when computing Cholesky decomposition: {e}. Using symeig method.",
+                    NumericalWarning,
                 )
                 method = "symeig"
 
         if method == "pivoted_cholesky":
-            root = pivoted_cholesky(self.evaluate(), max_iter=self._root_decomposition_size())
-        elif method == "symeig":
+            return RootLazyTensor(lazify(self.evaluate()).pivoted_cholesky(rank=self._root_decomposition_size()))
+        if method == "symeig":
             evals, evecs = self.symeig(eigenvectors=True)
             # TODO: only use non-zero evals (req. dealing w/ batches...)
             root = evecs * evals.clamp_min(0.0).sqrt().unsqueeze(-2)
@@ -1661,8 +1786,8 @@ class LazyTensor(ABC):
         This can be used for sampling from a Gaussian distribution, or for obtaining a
         low-rank version of a matrix
         """
-        from .root_lazy_tensor import RootLazyTensor
         from .non_lazy_tensor import lazify
+        from .root_lazy_tensor import RootLazyTensor
 
         if not self.is_square:
             raise RuntimeError(
@@ -1856,25 +1981,31 @@ class LazyTensor(ABC):
             pass
         return self._symeig(eigenvectors=eigenvectors)
 
-    def to(self, device_id):
+    def to(self, *args, **kwargs):
         """
-        A device-agnostic method of moving the lazy_tensor to the specified device.
+        A device-agnostic method of moving the lazy_tensor to the specified device or dtype.
+        Note that we do NOT support non_blocking or other `torch.to` options other than
+        device and dtype and these options will be silently ignored.
 
         Args:
-            device_id (:obj: `torch.device`): Which device to use (GPU or CPU).
+            device (:obj: `torch.device`): Which device to use (GPU or CPU).
+            dtype (:obj: `torch.dtype`): Which dtype to use (double, float, or half).
         Returns:
             :obj:`~gpytorch.lazy.LazyTensor`: New LazyTensor identical to self on specified device
         """
+
+        device, dtype = _to_helper(*args, **kwargs)
+
         new_args = []
         new_kwargs = {}
         for arg in self._args:
             if hasattr(arg, "to"):
-                new_args.append(arg.to(device_id))
+                new_args.append(arg.to(dtype=dtype, device=device))
             else:
                 new_args.append(arg)
         for name, val in self._kwargs.items():
             if hasattr(val, "to"):
-                new_kwargs[name] = val.to(device_id)
+                new_kwargs[name] = val.to(dtype=dtype, device=device)
             else:
                 new_kwargs[name] = val
         return self.__class__(*new_args, **new_kwargs)
@@ -1924,6 +2055,32 @@ class LazyTensor(ABC):
 
         return res
 
+    def type(self, dtype):
+        """
+        This method operates similarly to :func:`torch.Tensor.type`.
+        """
+        attr_flag = _TYPES_DICT[dtype]
+
+        new_args = []
+        new_kwargs = {}
+        for arg in self._args:
+            if hasattr(arg, attr_flag):
+                try:
+                    new_args.append(arg.clone().to(dtype))
+                except AttributeError:
+                    new_args.append(deepcopy(arg).to(dtype))
+            else:
+                new_args.append(arg)
+        for name, val in self._kwargs.items():
+            if hasattr(val, attr_flag):
+                try:
+                    new_kwargs[name] = val.clone().to(dtype)
+                except AttributeError:
+                    new_kwargs[name] = deepcopy(val).to(dtype)
+            else:
+                new_kwargs[name] = val
+        return self.__class__(*new_args, **new_kwargs)
+
     def unsqueeze(self, dim):
         positive_dim = (self.dim() + dim + 1) if dim < 0 else dim
         if positive_dim > len(self.batch_shape):
@@ -1953,7 +2110,11 @@ class LazyTensor(ABC):
 
         if settings.ciq_samples.on():
             base_samples = torch.randn(
-                *self.batch_shape, self.size(-1), num_samples, dtype=self.dtype, device=self.device
+                *self.batch_shape,
+                self.size(-1),
+                num_samples,
+                dtype=self.dtype,
+                device=self.device,
             )
             base_samples = base_samples.permute(-1, *range(self.dim() - 1)).contiguous()
             base_samples = base_samples.unsqueeze(-1)
@@ -1973,7 +2134,11 @@ class LazyTensor(ABC):
                 covar_root = self.root_decomposition().root
 
             base_samples = torch.randn(
-                *self.batch_shape, covar_root.size(-1), num_samples, dtype=self.dtype, device=self.device
+                *self.batch_shape,
+                covar_root.size(-1),
+                num_samples,
+                dtype=self.dtype,
+                device=self.device,
             )
             samples = covar_root.matmul(base_samples).permute(-1, *range(self.dim() - 1)).contiguous()
 
@@ -1992,13 +2157,14 @@ class LazyTensor(ABC):
             :obj:`gpytorch.lazy.SumLazyTensor`:
                 A sum lazy tensor representing the sum of this lazy tensor and other.
         """
+        from torch import Tensor
+
+        from .added_diag_lazy_tensor import AddedDiagLazyTensor
+        from .diag_lazy_tensor import DiagLazyTensor
+        from .non_lazy_tensor import lazify
+        from .root_lazy_tensor import RootLazyTensor
         from .sum_lazy_tensor import SumLazyTensor
         from .zero_lazy_tensor import ZeroLazyTensor
-        from .diag_lazy_tensor import DiagLazyTensor
-        from .added_diag_lazy_tensor import AddedDiagLazyTensor
-        from .root_lazy_tensor import RootLazyTensor
-        from .non_lazy_tensor import lazify
-        from torch import Tensor
 
         if isinstance(other, ZeroLazyTensor):
             return self
@@ -2094,9 +2260,11 @@ class LazyTensor(ABC):
         # Alternatively, if we're using tensor indices and losing dimensions, use self._get_indices
         if row_col_are_absorbed:
             # Convert all indices into tensor indices
-            (*batch_indices, row_index, col_index,) = _convert_indices_to_tensors(
-                self, (*batch_indices, row_index, col_index)
-            )
+            (
+                *batch_indices,
+                row_index,
+                col_index,
+            ) = _convert_indices_to_tensors(self, (*batch_indices, row_index, col_index))
             res = self._get_indices(row_index, col_index, *batch_indices)
         else:
             res = self._getitem(row_index, col_index, *batch_indices)
@@ -2141,10 +2309,11 @@ class LazyTensor(ABC):
         if settings.verbose_linalg.on():
             settings.verbose_linalg.logger.debug(f"Running symeig on a matrix of size {self.shape}.")
 
-        dtype = self.dtype  # perform decomposition in double precision for numerical stability
-        # TODO: Use fp64 registry once #1213 is addressed
-        evals, evecs = torch.linalg.eigh(self.evaluate().to(dtype=torch.double))
-        # chop any negative eigenvalues. TODO: warn if evals are significantly negative
+        # potentially perform decomposition in double precision for numerical stability
+        dtype = self.dtype
+        evals, evecs = torch.linalg.eigh(self.evaluate().to(dtype=settings._linalg_dtype_symeig.value()))
+        # chop any negative eigenvalues.
+        # TODO: warn if evals are significantly negative
         evals = evals.clamp_min(0.0).to(dtype=dtype)
         if eigenvectors:
             evecs = NonLazyTensor(evecs.to(dtype=dtype))
@@ -2155,6 +2324,9 @@ class LazyTensor(ABC):
     def __matmul__(self, other):
         return self.matmul(other)
 
+    def __rmatmul__(self, other: Tensor) -> Tensor:
+        return self.rmatmul(other)
+
     def __mul__(self, other):
         return self.mul(other)
 
