@@ -12,14 +12,15 @@ from ..lazy import (
     DiagLazyTensor,
     KroneckerProductDiagLazyTensor,
     KroneckerProductLazyTensor,
+    LazyEvaluatedKernelTensor,
     RootLazyTensor,
+    lazify,
 )
 from ..likelihoods import Likelihood, _GaussianLikelihoodBase
-from ..utils.pivoted_cholesky import pivoted_cholesky
 
 
 class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
-    """Base class for multi-task Gaussian Likelihoods, supporting general heteroskedastic noise models. """
+    """Base class for multi-task Gaussian Likelihoods, supporting general heteroskedastic noise models."""
 
     def __init__(self, num_tasks, noise_covar, rank=0, task_correlation_prior=None, batch_shape=torch.Size()):
         """
@@ -88,6 +89,10 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
         """
         mean, covar = function_dist.mean, function_dist.lazy_covariance_matrix
 
+        # ensure that sumKroneckerLT is actually called
+        if isinstance(covar, LazyEvaluatedKernelTensor):
+            covar = covar.evaluate_kernel()
+
         covar_kron_lt = self._shaped_noise_covar(mean.shape, add_noise=self.has_global_noise)
         covar = covar + covar_kron_lt
 
@@ -126,7 +131,7 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
 
     def forward(self, function_samples: Tensor, *params: Any, **kwargs: Any) -> base_distributions.Normal:
         noise = self._shaped_noise_covar(function_samples.shape, *params, **kwargs).diag()
-        noise = noise.view(*noise.shape[:-1], *function_samples.shape[-2:])
+        noise = noise.reshape(*noise.shape[:-1], *function_samples.shape[-2:])
         return base_distributions.Independent(base_distributions.Normal(function_samples, noise.sqrt()), 1)
 
 
@@ -249,7 +254,8 @@ class MultitaskGaussianLikelihood(_MultitaskGaussianLikelihoodBase):
         # internally uses a pivoted cholesky decomposition to construct a low rank
         # approximation of the covariance
         if self.rank > 0:
-            self.task_noise_covar_factor.data = pivoted_cholesky(value, max_iter=self.rank)
+            with torch.no_grad():
+                self.task_noise_covar_factor.data = lazify(value).pivoted_cholesky(rank=self.rank)
         else:
             raise AttributeError("Cannot set non-diagonal task noises when covariance is diagonal.")
 
