@@ -40,7 +40,17 @@ class DefaultPredictionStrategy(object):
         self._train_shape = train_prior_dist.event_shape
 
         # Flatten the training labels
-        train_labels = train_labels.reshape(*train_labels.shape[: -len(self.train_shape)], self._train_shape.numel())
+        try:
+            train_labels = train_labels.reshape(
+                *train_labels.shape[: -len(self.train_shape)], self._train_shape.numel()
+            )
+        except RuntimeError:
+            raise RuntimeError(
+                "Flattening the training labels failed. The most common cause of this error is "
+                + "that the shapes of the prior mean and the training labels are mismatched. "
+                + "The shape of the train targets is {0}, ".format(train_labels.shape)
+                + "while the reported shape of the mean is {0}.".format(train_prior_dist.mean.shape)
+            )
 
         self.train_inputs = train_inputs
         self.train_prior_dist = train_prior_dist
@@ -676,7 +686,7 @@ class RFFPredictionStrategy(DefaultPredictionStrategy):
             torch.eye(train_factor.size(-1), dtype=train_factor.dtype, device=train_factor.device)
             - (train_factor.transpose(-1, -2) @ train_train_covar.inv_matmul(train_factor)) * constant
         )
-        return psd_safe_cholesky(inner_term, jitter=settings.cholesky_jitter.value())
+        return psd_safe_cholesky(inner_term)
 
     def exact_prediction(self, joint_mean, joint_covar):
         # Find the components of the distribution that contain test data
