@@ -59,7 +59,9 @@ class _value_context:
         self._orig_value = self.__class__.value()
         self._instance_value = value
 
-    def __enter__(self,):
+    def __enter__(
+        self,
+    ):
         self.__class__._set_value(self._instance_value)
 
     def __exit__(self, *args):
@@ -102,9 +104,13 @@ class _dtype_value_context:
         self._orig_half_value = self.__class__.value()
         self._instance_half_value = half
 
-    def __enter__(self,):
+    def __enter__(
+        self,
+    ):
         self.__class__._set_value(
-            self._instance_float_value, self._instance_double_value, self._instance_half_value,
+            self._instance_float_value,
+            self._instance_double_value,
+            self._instance_half_value,
         )
 
     def __exit__(self, *args):
@@ -414,6 +420,21 @@ class max_cg_iterations(_value_context):
     _global_value = 1000
 
 
+class min_fixed_noise(_dtype_value_context):
+    """
+    The minimum noise value that can be used in :obj:`~gpytorch.likelihoods.FixedNoiseGaussianLikelihood`.
+    If the supplied noise values are smaller than this, they are rounded up and a warning is raised.
+
+    - Default for `float`: 1e-4
+    - Default for `double`: 1e-6
+    - Default for `half`: 1e-3
+    """
+
+    _global_float_value = 1e-4
+    _global_double_value = 1e-6
+    _global_half_value = 1e-3
+
+
 class min_variance(_dtype_value_context):
     """
     The minimum variance that can be returned from :obj:`~gpytorch.distributions.MultivariateNormal#variance`.
@@ -431,7 +452,7 @@ class min_variance(_dtype_value_context):
 
 class cholesky_jitter(_dtype_value_context):
     """
-    The jitter value passed to `psd_safe_cholesky` when using cholesky solves.
+    The jitter value used by `psd_safe_cholesky` when using cholesky solves.
 
     - Default for `float`: 1e-6
     - Default for `double`: 1e-8
@@ -452,6 +473,16 @@ class cholesky_jitter(_dtype_value_context):
         return super().value(dtype=dtype)
 
 
+class cholesky_max_tries(_value_context):
+    """
+    The max_tries value used by `psd_safe_cholesky` when using cholesky solves.
+
+    (Default: 3)
+    """
+
+    _global_value = 3
+
+
 class cg_tolerance(_value_context):
     """
     Relative residual tolerance to use for terminating CG.
@@ -640,6 +671,19 @@ class prior_mode(_feature_flag):
     _default = False
 
 
+class sgpr_diagonal_correction(_feature_flag):
+    """
+    If set to true, during posterior prediction the variances of the InducingPointKernel
+    will be corrected to match the variances of the exact kernel.
+
+    If false then no such correction will be performed (this is the default in other libraries).
+
+    (Default: True)
+    """
+
+    _default = True
+
+
 class skip_logdet_forward(_feature_flag):
     """
     .. warning:
@@ -658,7 +702,7 @@ class skip_logdet_forward(_feature_flag):
     pass will skip certain computations (i.e. the logdet computation), and will therefore
     be improper estimates.
 
-    If you're using SGD (or a varient) to optimize parameters, you probably
+    If you're using SGD (or a variant) to optimize parameters, you probably
     don't need an accurate MLL estimate; you only need accurate gradients. So
     this setting may give your model a performance boost.
 
@@ -668,6 +712,40 @@ class skip_logdet_forward(_feature_flag):
     _default = False
 
 
+class _linalg_dtype_symeig(_value_context):
+    _global_value = torch.double
+
+
+class _linalg_dtype_cholesky(_value_context):
+    _global_value = torch.double
+
+
+class linalg_dtypes:
+    """
+    Whether to perform less stable linalg calls in double precision or in a lower precision.
+    Currently, the default is to apply all symeig calls and cholesky calls within variational
+    methods in double precision.
+
+    (Default: torch.double)
+    """
+
+    def __init__(self, default=torch.double, symeig=None, cholesky=None):
+        symeig = default if symeig is None else symeig
+        cholesky = default if cholesky is None else cholesky
+
+        self.symeig = _linalg_dtype_symeig(symeig)
+        self.cholesky = _linalg_dtype_cholesky(cholesky)
+
+    def __enter__(self):
+        self.symeig.__enter__()
+        self.cholesky.__enter__()
+
+    def __exit__(self, *args):
+        self.symeig.__exit__()
+        self.cholesky.__exit__()
+        return False
+
+
 class terminate_cg_by_size(_feature_flag):
     """
     If set to true, cg will terminate after n iterations for an n x n matrix.
