@@ -9,7 +9,9 @@ from unittest.mock import MagicMock, patch
 import torch
 
 import gpytorch
-from gpytorch.utils.cholesky import CHOLESKY_METHOD
+from gpytorch.settings import linalg_dtypes
+from gpytorch.utils.errors import CachingError
+from gpytorch.utils.memoize import get_from_cache
 
 from .base_test_case import BaseTestCase
 
@@ -53,6 +55,22 @@ class RectangularLazyTensorTestCase(BaseTestCase):
             if arg_copy.requires_grad and arg_copy.is_leaf and arg_copy.grad is not None:
                 self.assertAllClose(arg.grad, arg_copy.grad, **self.tolerances["matmul"])
 
+    def _test_rmatmul(self, lhs):
+        lazy_tensor = self.create_lazy_tensor().detach().requires_grad_(True)
+        lazy_tensor_copy = lazy_tensor.clone().detach().requires_grad_(True)
+        evaluated = self.evaluate_lazy_tensor(lazy_tensor_copy)
+
+        res = lhs @ lazy_tensor
+        actual = lhs @ evaluated
+        self.assertAllClose(res, actual)
+
+        grad = torch.randn_like(res)
+        res.backward(gradient=grad)
+        actual.backward(gradient=grad)
+        for arg, arg_copy in zip(lazy_tensor.representation(), lazy_tensor_copy.representation()):
+            if arg_copy.requires_grad and arg_copy.is_leaf and arg_copy.grad is not None:
+                self.assertAllClose(arg.grad, arg_copy.grad, **self.tolerances["matmul"])
+
     def test_add(self):
         lazy_tensor = self.create_lazy_tensor()
         evaluated = self.evaluate_lazy_tensor(lazy_tensor)
@@ -70,20 +88,36 @@ class RectangularLazyTensorTestCase(BaseTestCase):
 
     def test_matmul_vec(self):
         lazy_tensor = self.create_lazy_tensor()
-        rhs = torch.randn(lazy_tensor.size(-1))
 
         # We skip this test if we're dealing with batch LazyTensors
         # They shouldn't multiply by a vec
         if lazy_tensor.ndimension() > 2:
             return
-        else:
-            return self._test_matmul(rhs)
+
+        rhs = torch.randn(lazy_tensor.size(-1))
+        return self._test_matmul(rhs)
+
+    def test_rmatmul_vec(self):
+        lazy_tensor = self.create_lazy_tensor()
+
+        # We skip this test if we're dealing with batch LazyTensors
+        # They shouldn't multiply by a vec
+        if lazy_tensor.ndimension() > 2:
+            return
+
+        lhs = torch.randn(lazy_tensor.size(-2))
+        return self._test_rmatmul(lhs)
 
     def test_matmul_matrix(self):
         lazy_tensor = self.create_lazy_tensor()
         rhs = torch.randn(*lazy_tensor.batch_shape, lazy_tensor.size(-1), 4)
         return self._test_matmul(rhs)
 
+    def test_rmatmul_matrix(self):
+        lazy_tensor = self.create_lazy_tensor()
+        lhs = torch.randn(*lazy_tensor.batch_shape, 4, lazy_tensor.size(-2))
+        return self._test_rmatmul(lhs)
+
     def test_matmul_matrix_broadcast(self):
         lazy_tensor = self.create_lazy_tensor()
 
@@ -103,6 +137,25 @@ class RectangularLazyTensorTestCase(BaseTestCase):
             rhs = torch.randn(*batch_shape, lazy_tensor.size(-1), 4)
             self._test_matmul(rhs)
 
+    def test_rmatmul_matrix_broadcast(self):
+        lazy_tensor = self.create_lazy_tensor()
+
+        # Left hand size has one more batch dimension
+        batch_shape = torch.Size((3, *lazy_tensor.batch_shape))
+        lhs = torch.randn(*batch_shape, 4, lazy_tensor.size(-2))
+        self._test_rmatmul(lhs)
+
+        if lazy_tensor.ndimension() > 2:
+            # Left hand size has one fewer batch dimension
+            batch_shape = torch.Size(lazy_tensor.batch_shape[1:])
+            lhs = torch.randn(*batch_shape, 4, lazy_tensor.size(-2))
+            self._test_rmatmul(lhs)
+
+            # Left hand size has a singleton dimension
+            batch_shape = torch.Size((*lazy_tensor.batch_shape[:-1], 1))
+            lhs = torch.randn(*batch_shape, 4, lazy_tensor.size(-2))
+            self._test_rmatmul(lhs)
+
     def test_constant_mul(self):
         lazy_tensor = self.create_lazy_tensor()
         evaluated = self.evaluate_lazy_tensor(lazy_tensor)
@@ -208,15 +261,28 @@ class RectangularLazyTensorTestCase(BaseTestCase):
         # Batch case
         else:
             for batch_index in product(
-                [torch.tensor([0, 1, 1, 0]), slice(None, None, None)], repeat=(lazy_tensor.dim() - 2)
+                [torch.tensor([0, 1, 1, 0]), slice(None, None, None)],
+                repeat=(lazy_tensor.dim() - 2),
             ):
-                index = (*batch_index, torch.tensor([0, 1, 0, 2]), torch.tensor([1, 2, 0, 1]))
+                index = (
+                    *batch_index,
+                    torch.tensor([0, 1, 0, 2]),
+                    torch.tensor([1, 2, 0, 1]),
+                )
                 res, actual = lazy_tensor[index], evaluated[index]
                 self.assertAllClose(res, actual)
-                index = (*batch_index, torch.tensor([0, 1, 0, 2]), slice(None, None, None))
+                index = (
+                    *batch_index,
+                    torch.tensor([0, 1, 0, 2]),
+                    slice(None, None, None),
+                )
                 res, actual = gpytorch.delazify(lazy_tensor[index]), evaluated[index]
                 self.assertAllClose(res, actual)
-                index = (*batch_index, slice(None, None, None), torch.tensor([0, 1, 2, 1]))
+                index = (
+                    *batch_index,
+                    slice(None, None, None),
+                    torch.tensor([0, 1, 2, 1]),
+                )
                 res, actual = gpytorch.delazify(lazy_tensor[index]), evaluated[index]
                 self.assertAllClose(res, actual)
                 index = (*batch_index, slice(None, None, None), slice(None, None, None))
@@ -294,8 +360,11 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
         "root_decomposition": {"rtol": 0.05},
         "root_inv_decomposition": {"rtol": 0.05, "atol": 0.02},
         "sample": {"rtol": 0.3, "atol": 0.3},
-        "sqrt_inv_matmul": {"rtol": 1e-4, "atol": 1e-3},
-        "symeig": {"rtol": 1e-4, "atol": 1e-3},
+        "sqrt_inv_matmul": {"rtol": 1e-2, "atol": 1e-3},
+        "symeig": {
+            "double": {"rtol": 1e-4, "atol": 1e-3},
+            "float": {"rtol": 1e-3, "atol": 1e-2},
+        },
         "svd": {"rtol": 1e-4, "atol": 1e-3},
     }
 
@@ -357,10 +426,10 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
                 with gpytorch.settings.num_trace_samples(256), gpytorch.settings.max_cholesky_size(
                     math.inf if cholesky else 0
                 ), gpytorch.settings.cg_tolerance(1e-5):
-
-                    res_inv_quad, res_logdet = lazy_tensor.inv_quad_logdet(
-                        inv_quad_rhs=vecs, logdet=True, reduce_inv_quad=reduce_inv_quad
-                    )
+                    with gpytorch.settings.min_preconditioning_size(4), gpytorch.settings.max_preconditioner_size(2):
+                        res_inv_quad, res_logdet = lazy_tensor.inv_quad_logdet(
+                            inv_quad_rhs=vecs, logdet=True, reduce_inv_quad=reduce_inv_quad
+                        )
 
             actual_inv_quad = evaluated.inverse().matmul(vecs_copy).mul(vecs_copy).sum(-2)
             if reduce_inv_quad:
@@ -456,13 +525,19 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
             root_rhs = new_lt.root_decomposition().matmul(rhs)
             self.assertAllClose(root_rhs, concat_rhs, **self.tolerances["root_decomposition"])
 
+            # check that root inv is cached
+            root_inv = get_from_cache(new_lt, "root_inv_decomposition")
             # check that the inverse root decomposition is close
             concat_solve = torch.linalg.solve(concatenated_lt, rhs.unsqueeze(-1)).squeeze(-1)
-            root_inv_solve = new_lt.root_inv_decomposition().matmul(rhs)
+            root_inv_solve = root_inv.matmul(rhs)
             self.assertLess(
                 (root_inv_solve - concat_solve).norm() / concat_solve.norm(),
                 self.tolerances["root_inv_decomposition"]["rtol"],
             )
+            # test generate_inv_roots=False
+            new_lt = lazy_tensor.cat_rows(new_rows, new_point, generate_inv_roots=False)
+            with self.assertRaises(CachingError):
+                get_from_cache(new_lt, "root_inv_decomposition")
 
     def test_cholesky(self):
         lazy_tensor = self.create_lazy_tensor()
@@ -475,6 +550,14 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
             self.assertAllClose(res, actual, **self.tolerances["cholesky"])
             # TODO: Check gradients
 
+    def test_double(self):
+        lazy_tensor = self.create_lazy_tensor()
+        evaluated = self.evaluate_lazy_tensor(lazy_tensor)
+
+        res = lazy_tensor.double()
+        actual = evaluated.double()
+        self.assertEqual(res.dtype, actual.dtype)
+
     def test_diag(self):
         lazy_tensor = self.create_lazy_tensor()
         evaluated = self.evaluate_lazy_tensor(lazy_tensor)
@@ -484,6 +567,25 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
         actual = actual.view(*lazy_tensor.batch_shape, -1)
         self.assertAllClose(res, actual, **self.tolerances["diag"])
 
+    def test_float(self):
+        lazy_tensor = self.create_lazy_tensor().double()
+        evaluated = self.evaluate_lazy_tensor(lazy_tensor)
+
+        res = lazy_tensor.float()
+        actual = evaluated.float()
+        self.assertEqual(res.dtype, actual.dtype)
+
+    def _test_half(self, lazy_tensor):
+        evaluated = self.evaluate_lazy_tensor(lazy_tensor)
+
+        res = lazy_tensor.half()
+        actual = evaluated.half()
+        self.assertEqual(res.dtype, actual.dtype)
+
+    def test_half(self):
+        lazy_tensor = self.create_lazy_tensor()
+        self._test_half(lazy_tensor)
+
     def test_inv_matmul_vector(self, cholesky=False):
         lazy_tensor = self.create_lazy_tensor()
         rhs = torch.randn(lazy_tensor.size(-1))
@@ -614,15 +716,13 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
                 chol = lazy_tensor.root_decomposition().root.clone()
                 gpytorch.utils.memoize.clear_cache_hook(lazy_tensor)
                 gpytorch.utils.memoize.add_to_cache(
-                    lazy_tensor, "root_decomposition", gpytorch.lazy.RootLazyTensor(chol)
+                    lazy_tensor,
+                    "root_decomposition",
+                    gpytorch.lazy.RootLazyTensor(chol),
                 )
 
-                _wrapped_cholesky = MagicMock(
-                    wraps=torch.linalg.cholesky
-                    if CHOLESKY_METHOD == "torch.linalg.cholesky"
-                    else torch.linalg.cholesky_ex
-                )
-                with patch(CHOLESKY_METHOD, new=_wrapped_cholesky) as cholesky_mock:
+                _wrapped_cholesky = MagicMock(wraps=torch.linalg.cholesky_ex)
+                with patch("torch.linalg.cholesky_ex", new=_wrapped_cholesky) as cholesky_mock:
                     self._test_inv_quad_logdet(reduce_inv_quad=True, cholesky=True, lazy_tensor=lazy_tensor)
                 self.assertFalse(cholesky_mock.called)
 
@@ -670,7 +770,7 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
         # Perform forward pass
         with gpytorch.settings.max_cg_iterations(200):
             sqrt_inv_matmul_res, inv_quad_res = lazy_tensor.sqrt_inv_matmul(rhs, lhs)
-        evals, evecs = evaluated.symeig(eigenvectors=True)
+        evals, evecs = torch.linalg.eigh(evaluated)
         matrix_inv_root = evecs @ (evals.sqrt().reciprocal().unsqueeze(-1) * evecs.transpose(-1, -2))
         sqrt_inv_matmul_actual = lhs_copy @ matrix_inv_root @ rhs_copy
         inv_quad_actual = (lhs_copy @ matrix_inv_root).pow(2).sum(dim=-1)
@@ -708,7 +808,7 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
         # Perform forward pass
         with gpytorch.settings.max_cg_iterations(200):
             sqrt_inv_matmul_res = lazy_tensor.sqrt_inv_matmul(rhs)
-        evals, evecs = evaluated.symeig(eigenvectors=True)
+        evals, evecs = torch.linalg.eigh(evaluated)
         matrix_inv_root = evecs @ (evals.sqrt().reciprocal().unsqueeze(-1) * evecs.transpose(-1, -2))
         sqrt_inv_matmul_actual = matrix_inv_root @ rhs_copy
 
@@ -727,51 +827,60 @@ class LazyTensorTestCase(RectangularLazyTensorTestCase):
                 self.assertAllClose(arg.grad, arg_copy.grad, **self.tolerances["sqrt_inv_matmul"])
 
     def test_symeig(self):
-        lazy_tensor = self.create_lazy_tensor().detach().requires_grad_(True)
-        lazy_tensor_copy = lazy_tensor.clone().detach().requires_grad_(True)
-        evaluated = self.evaluate_lazy_tensor(lazy_tensor_copy)
+        dtypes = {"double": torch.double, "float": torch.float}
+        for name, dtype in dtypes.items():
+            tolerances = self.tolerances["symeig"][name]
 
-        # Perform forward pass
-        evals_unsorted, evecs_unsorted = lazy_tensor.symeig(eigenvectors=True)
-        evecs_unsorted = evecs_unsorted.evaluate()
+            lazy_tensor = self.create_lazy_tensor().detach().requires_grad_(True)
+            lazy_tensor_copy = lazy_tensor.clone().detach().requires_grad_(True)
+            evaluated = self.evaluate_lazy_tensor(lazy_tensor_copy)
 
-        # since LazyTensor.symeig does not sort evals, we do this here for the check
-        evals, idxr = torch.sort(evals_unsorted, dim=-1, descending=False)
-        evecs = torch.gather(evecs_unsorted, dim=-1, index=idxr.unsqueeze(-2).expand(evecs_unsorted.shape))
+            # Perform forward pass
+            with linalg_dtypes(dtype):
+                evals_unsorted, evecs_unsorted = lazy_tensor.symeig(eigenvectors=True)
+                evecs_unsorted = evecs_unsorted.evaluate()
 
-        evals_actual, evecs_actual = torch.linalg.eigh(evaluated.double())
-        evals_actual = evals_actual.to(dtype=evaluated.dtype)
-        evecs_actual = evecs_actual.to(dtype=evaluated.dtype)
+            # since LazyTensor.symeig does not sort evals, we do this here for the check
+            evals, idxr = torch.sort(evals_unsorted, dim=-1, descending=False)
+            evecs = torch.gather(
+                evecs_unsorted,
+                dim=-1,
+                index=idxr.unsqueeze(-2).expand(evecs_unsorted.shape),
+            )
 
-        # Check forward pass
-        self.assertAllClose(evals, evals_actual, **self.tolerances["symeig"])
-        lt_from_eigendecomp = evecs @ torch.diag_embed(evals) @ evecs.transpose(-1, -2)
-        self.assertAllClose(lt_from_eigendecomp, evaluated, **self.tolerances["symeig"])
+            evals_actual, evecs_actual = torch.linalg.eigh(evaluated.type(dtype))
+            evals_actual = evals_actual.to(dtype=evaluated.dtype)
+            evecs_actual = evecs_actual.to(dtype=evaluated.dtype)
 
-        # if there are repeated evals, we'll skip checking the eigenvectors for those
-        any_evals_repeated = False
-        evecs_abs, evecs_actual_abs = evecs.abs(), evecs_actual.abs()
-        for idx in itertools.product(*[range(b) for b in evals_actual.shape[:-1]]):
-            eval_i = evals_actual[idx]
-            if torch.unique(eval_i.detach()).shape[-1] == eval_i.shape[-1]:  # detach to avoid pytorch/pytorch#41389
-                self.assertAllClose(evecs_abs[idx], evecs_actual_abs[idx], **self.tolerances["symeig"])
-            else:
-                any_evals_repeated = True
+            # Check forward pass
+            self.assertAllClose(evals, evals_actual, **tolerances)
+            lt_from_eigendecomp = evecs @ torch.diag_embed(evals) @ evecs.transpose(-1, -2)
+            self.assertAllClose(lt_from_eigendecomp, evaluated, **tolerances)
 
-        # Perform backward pass
-        symeig_grad = torch.randn_like(evals)
-        ((evals * symeig_grad).sum()).backward()
-        ((evals_actual * symeig_grad).sum()).backward()
+            # if there are repeated evals, we'll skip checking the eigenvectors for those
+            any_evals_repeated = False
+            evecs_abs, evecs_actual_abs = evecs.abs(), evecs_actual.abs()
+            for idx in itertools.product(*[range(b) for b in evals_actual.shape[:-1]]):
+                eval_i = evals_actual[idx]
+                if torch.unique(eval_i.detach()).shape[-1] == eval_i.shape[-1]:  # detach to avoid pytorch/pytorch#41389
+                    self.assertAllClose(evecs_abs[idx], evecs_actual_abs[idx], **tolerances)
+                else:
+                    any_evals_repeated = True
 
-        # Check grads if there were no repeated evals
-        if not any_evals_repeated:
-            for arg, arg_copy in zip(lazy_tensor.representation(), lazy_tensor_copy.representation()):
-                if arg_copy.requires_grad and arg_copy.is_leaf and arg_copy.grad is not None:
-                    self.assertAllClose(arg.grad, arg_copy.grad, **self.tolerances["symeig"])
+            # Perform backward pass
+            symeig_grad = torch.randn_like(evals)
+            ((evals * symeig_grad).sum()).backward()
+            ((evals_actual * symeig_grad).sum()).backward()
 
-        # Test with eigenvectors=False
-        _, evecs = lazy_tensor.symeig(eigenvectors=False)
-        self.assertIsNone(evecs)
+            # Check grads if there were no repeated evals
+            if not any_evals_repeated:
+                for arg, arg_copy in zip(lazy_tensor.representation(), lazy_tensor_copy.representation()):
+                    if arg_copy.requires_grad and arg_copy.is_leaf and arg_copy.grad is not None:
+                        self.assertAllClose(arg.grad, arg_copy.grad, **tolerances)
+
+            # Test with eigenvectors=False
+            _, evecs = lazy_tensor.symeig(eigenvectors=False)
+            self.assertIsNone(evecs)
 
     def test_svd(self):
         lazy_tensor = self.create_lazy_tensor().detach().requires_grad_(True)
