@@ -1,16 +1,17 @@
 #!/usr/bin/env python3
 
-from . import broadcasting, cholesky, fft, grid, interpolation, lanczos, pivoted_cholesky, quadrature, sparse, warnings
+from . import broadcasting, cholesky, grid, interpolation, lanczos, permutation, quadrature, sparse, warnings
 from .contour_integral_quad import contour_integral_quad
 from .linear_cg import linear_cg
 from .memoize import cached
 from .minres import minres
+from .pinverse import stable_pinverse
+from .qr import stable_qr
 from .stochastic_lq import StochasticLQ
 
 
 def prod(items):
-    """
-    """
+    """ """
     if len(items):
         res = items[0]
         for item in items[1:]:
@@ -27,14 +28,16 @@ __all__ = [
     "linear_cg",
     "StochasticLQ",
     "cholesky",
-    "fft",
     "grid",
     "interpolation",
     "lanczos",
     "minres",
-    "pivoted_cholesky",
+    "permutation",
+    "pinverse",
     "prod",
     "quadrature",
     "sparse",
+    "stable_pinverse",
+    "stable_qr",
     "warnings",
 ]
